@@ -1,4 +1,4 @@
-# $Id: W3C.pm 704 2007-08-28 21:14:05Z struan $
+# $Id: W3C.pm 738 2008-08-24 19:48:17Z struan $
 
 package WebService::Validator::HTML::W3C;
 
@@ -16,7 +16,7 @@ __PACKAGE__->mk_accessors(
 
 use vars qw( $VERSION $VALIDATOR_URI $HTTP_TIMEOUT );
 
-$VERSION       = 0.22;
+$VERSION       = 0.23;
 $VALIDATOR_URI = 'http://validator.w3.org/check';
 $HTTP_TIMEOUT  = 30;
 
@@ -37,7 +37,7 @@ WebService::Validator::HTML::W3C - Access the W3Cs online HTML validator
             printf ("%s is valid\n", $v->uri);
         } else {
             printf ("%s is not valid\n", $v->uri);
-            foreach $error ( @{$v->errors} ) {
+            foreach my $error ( @{$v->errors} ) {
                 printf("%s at line %d\n", $error->msg,
                                           $error->line);
             }
@@ -93,9 +93,9 @@ An HTTP proxy to use when communicating with the validation service.
 
 Controls which output format is used. Can be either xml or soap12.
 
-At the moment the default is XML as this is the only one supported by the Validator. However in the future it is moving to using SOAP for the detailed reporting.
+The default is soap12 as the XML format is deprecated and is likely to be removed in the future.
 
-The default will always work so unless you're using a development version of the Validator you can safely ignore this.
+The default will always work so unless you're using a locally installed Validator you can safely ignore this.
 
 =back 
 
@@ -118,7 +118,7 @@ sub _init {
     $self->validator_uri( $args{validator_uri} || $VALIDATOR_URI );
     $self->ua( $args{ua} );
     $self->_http_method( $args{detailed} ? 'GET' : 'HEAD' );
-    $self->_output( $args{output} || 'xml' );
+    $self->_output( $args{output} || 'soap12' );
     $self->proxy( $args{proxy} || '' );
 }
 
@@ -141,6 +141,14 @@ Validate a file by uploading it to the W3C Validator. NB This has only been test
 
 Validate a scalar containing HTML. 
 
+=head2 Alternate interface
+
+You can also pass a hash in to specify what you wish to validate. This is provided to ensure compatability with the CSS validator module.
+
+	$v->validate( uri => 'http://example.com/' );
+	$v->validate( string => $markup );
+	$v->validate( file => './file.html' );
+	
 =cut
 
 sub validate_file {
@@ -165,7 +173,23 @@ sub validate_markup {
 
 sub validate {
     my $self = shift;
-    my $uri = shift;
+
+	my ( %opts, $uri );
+	if ( scalar( @_ ) > 1 ) {
+		%opts = @_;
+		
+		if ( $opts{ 'uri' } ) {
+			$uri = $opts{ 'uri' };	
+		} elsif ( $opts{ 'string' } ) {
+			return $self->validate_markup( $opts{ 'string' } );
+		} elsif( $opts{ 'file' } ) {
+			return $self->validate_file( $opts{ 'file' } );
+		} else {
+			return self->validator_error( "You need to provide a uri, string or file to validate" );
+		}
+	} else {
+	    $uri = shift;		
+	}
 
     return $self->validator_error("You need to supply a URI to validate")
       unless $uri;
@@ -241,6 +265,10 @@ Returns the URI of the last page on which validation succeeded.
 
 Returns the number of errors that the validator encountered.
 
+=head2 errorcount
+
+Synonym for num_errors. There to match CSS Validator interface.
+
 =head2 errors
 
     $errors = $v->errors();
@@ -318,9 +346,11 @@ sub errors {
 
        foreach my $msg ( @messages ) {
            my $err = WebService::Validator::HTML::W3C::Error->new({ 
-                          line => $xp->find( './m:line', $msg )->get_node(1)->getChildNode(1)->getValue,
-                          col  => $xp->find( './m:col', $msg )->get_node(1)->getChildNode(1)->getValue,
-                          msg  => $xp->find( './m:message', $msg )->get_node(1)->getChildNode(1)->getValue,
+                          line          => $xp->find( './m:line', $msg )->get_node(1)->getChildNode(1)->getValue,
+                          col           => $xp->find( './m:col', $msg )->get_node(1)->getChildNode(1)->getValue,
+                          msg           => $xp->find( './m:message', $msg )->get_node(1)->getChildNode(1)->getValue,
+                          msgid         => $xp->find( './m:messageid', $msg )->get_node(1)->getChildNode(1)->getValue,
+                          explanation   => $xp->find( './m:explanation', $msg )->get_node(1)->getChildNode(1)->getValue,
                       });
 
             push @errs, $err;
@@ -330,6 +360,10 @@ sub errors {
     return \@errs;
 }
 
+sub errorcount {
+	shift->num_errors;
+}
+
 sub warnings {
     my $self = shift;
 
@@ -427,6 +461,11 @@ isn't in the expected format then you'll get this error. Most likely to
 happen if you ask for SOAP output from a validator that doesn't
 support that format.
 
+=item You need to provide a uri, string or file to validate
+
+You've passed in a hash ( or in fact more than one argument ) to validate
+but the hash does not contain one of the three expected keys.
+
 =back
 
 =cut
@@ -541,6 +580,9 @@ the W3Cs DTDs. You have to fetch the relevant DTDs and so on.
 There is also the L<HTML::Parser> based L<HTML::Lint> which mostly checks for 
 known tags rather than XML/HTML validity.
 
+L<WebService::Validator::CSS::W3C> provides the same functionality as this module
+for the W3C's CSS validator. 
+
 =head1 IMPORTANT
 
 This module is not in any way associated with the W3C so please do not 
@@ -568,6 +610,9 @@ TEST_AUTHOR.
 That said I'm very happy to hear about bugs. All the more so if they come
 with patches ;).
 
+Please use http://rt.cpan.org/ for filing bug reports, and indeed feature
+requests.
+
 =head1 THANKS
 
 To the various people on the code review ladder mailing list who 
@@ -577,6 +622,8 @@ Carl Vincent provided a patch to allow for proxy support.
 
 Chris Dolan provided a patch to allow for custom user agents.
 
+Matt Ryder provided a patch for support of the explanations in the SOAP output. 
+
 =head1 SUPPORT
 
 author email or via http://rt.cpan.org/.
@@ -589,7 +636,9 @@ L<http://www.exo.org.uk/code/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2003-2005 Struan Donald. All rights reserved.
+Copyright (C) 2003-2008 Struan Donald. All rights reserved.
+
+=head1 LICENSE
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
