@@ -1,11 +1,11 @@
-# $Id: Error.pm 32 2003-11-26 13:52:18Z struan $
+# $Id: Error.pm 734 2008-08-16 16:56:05Z struan $
 
 package WebService::Validator::HTML::W3C::Error;
 
 use strict;
 use base qw(Class::Accessor);
 
-__PACKAGE__->mk_accessors( qw( line col msg ) );
+__PACKAGE__->mk_accessors( qw( line col msg msgid explanation ) );
 
 1;
 
@@ -21,6 +21,8 @@ This is a wee internal module for WebService::Validator::HTML::W3C. It has
 three methods: line, col and msg which return the line number, column number 
 and the error that occured at that location in a validated page.
 
+If you are using the soap output then you get additional information in the msgid and explanation methods.
+
 =head1 SEE ALSO
 
 L<WebService::Validator::HTML::W3C>
